@@ -9,28 +9,28 @@
 # lost the next time ruby is built.
 
 module RbConfig
-  RUBY_VERSION.start_with?("3.0.") or
-    raise "ruby lib version (3.0.6) doesn't match executable version (#{RUBY_VERSION})"
+  RUBY_VERSION.start_with?("3.1.") or
+    raise "ruby lib version (3.1.1) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
-  TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.0/sparc64-openbsd")
+  TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.1/sparc64-openbsd")
   # DESTDIR on make install.
   DESTDIR = '' unless defined? DESTDIR
   # The hash configurations stored.
   CONFIG = {}
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
-  CONFIG["MINOR"] = "0"
-  CONFIG["TEENY"] = "6"
-  CONFIG["PATCHLEVEL"] = "216"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.0.6/bin/install -c'
+  CONFIG["MINOR"] = "1"
+  CONFIG["TEENY"] = "1"
+  CONFIG["PATCHLEVEL"] = "18"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.1.1/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
-  CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)30"
-  CONFIG["RUBY_INSTALL_NAME"] = "$(RUBY_BASE_NAME)30"
-  CONFIG["RUBY_SO_NAME"] = "ruby30"
+  CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)31"
+  CONFIG["RUBY_INSTALL_NAME"] = "$(RUBY_BASE_NAME)31"
+  CONFIG["RUBY_SO_NAME"] = "ruby31"
   CONFIG["exec"] = "exec"
-  CONFIG["ruby_pc"] = "ruby-3.0.pc"
+  CONFIG["ruby_pc"] = "ruby-3.1.pc"
   CONFIG["CC_WRAPPER"] = ""
   CONFIG["PACKAGE"] = "ruby"
   CONFIG["BUILTIN_TRANSSRCS"] = " enc/trans/newline.c"
@@ -44,7 +44,7 @@ module RbConfig
   CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=30' '--with-soname=ruby30' '--with-ruby-version=3.0' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CXXFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CFLAGS=-O2 -pipe'"
+  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=31' '--with-soname=ruby31' '--with-ruby-version=minor' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe'"
   CONFIG["CONFIGURE"] = "configure"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
@@ -54,7 +54,7 @@ module RbConfig
   CONFIG["sitedir"] = "$(rubylibprefix)/site_ruby"
   CONFIG["rubyarchdir"] = "$(rubylibdir)/$(arch)"
   CONFIG["rubylibdir"] = "$(rubylibprefix)/$(ruby_version)"
-  CONFIG["ruby_version"] = "3.0"
+  CONFIG["ruby_version"] = "3.1"
   CONFIG["sitearch"] = "$(arch)"
   CONFIG["arch"] = "sparc64-openbsd"
   CONFIG["sitearchincludedir"] = "$(includedir)/$(sitearch)"
@@ -63,8 +63,8 @@ module RbConfig
   CONFIG["archlibdir"] = "$(libdir)/$(arch)"
   CONFIG["libdirname"] = "libdir"
   CONFIG["RUBY_EXEC_PREFIX"] = "/usr/local"
-  CONFIG["RUBY_LIB_VERSION"] = "3.0"
-  CONFIG["RUBY_LIB_VERSION_STYLE"] = ""
+  CONFIG["RUBY_LIB_VERSION"] = ""
+  CONFIG["RUBY_LIB_VERSION_STYLE"] = "2\t/* minor */"
   CONFIG["RI_BASE_NAME"] = "ri"
   CONFIG["ridir"] = "$(datarootdir)/$(RI_BASE_NAME)"
   CONFIG["rubysitearchprefix"] = "$(rubylibprefix)/$(sitearch)"
@@ -80,7 +80,7 @@ module RbConfig
   CONFIG["MAINLIBS"] = "-lz -pthread -lgmp -lm "
   CONFIG["ENABLE_SHARED"] = "yes"
   CONFIG["DLDSHARED"] = "$(CC) -shared $(CCDLFLAGS)"
-  CONFIG["DLDLIBS"] = " -lc"
+  CONFIG["DLDLIBS"] = "-lc"
   CONFIG["SOLIBS"] = "$(MAINLIBS)"
   CONFIG["LIBRUBYARG_SHARED"] = "-Wl,-rpath,$(libdir) -L$(libdir) -l$(RUBY_SO_NAME)"
   CONFIG["LIBRUBYARG_STATIC"] = "-Wl,-rpath,$(libdir) -L$(libdir) -l$(RUBY_SO_NAME)-static $(MAINLIBS)"
@@ -95,9 +95,9 @@ module RbConfig
   CONFIG["EXTDLDFLAGS"] = ""
   CONFIG["EXTLDFLAGS"] = ""
   CONFIG["strict_warnflags"] = ""
-  CONFIG["warnflags"] = "-Wall -Wextra -Wdeprecated-declarations -Wimplicit-function-declaration -Wimplicit-int -Wpointer-arith -Wwrite-strings -Wmissing-noreturn -Wno-cast-function-type -Wno-constant-logical-operand -Wno-long-long -Wno-missing-field-initializers -Wno-overlength-strings -Wno-packed-bitfield-compat -Wno-parentheses-equality -Wno-self-assign -Wno-tautological-compare -Wno-unused-parameter -Wno-unused-value -Wunused-variable"
+  CONFIG["warnflags"] = "-Wall -Wextra -Wdeprecated-declarations -Wimplicit-function-declaration -Wimplicit-int -Wpointer-arith -Wwrite-strings -Wmissing-noreturn -Wno-cast-function-type -Wno-constant-logical-operand -Wno-long-long -Wno-missing-field-initializers -Wno-overlength-strings -Wno-packed-bitfield-compat -Wno-parentheses-equality -Wno-self-assign -Wno-tautological-compare -Wno-unused-parameter -Wno-unused-value -Wunused-variable -Wundef"
   CONFIG["debugflags"] = "-ggdb3"
-  CONFIG["optflags"] = "-O3"
+  CONFIG["optflags"] = "-O3 -fno-fast-math"
   CONFIG["NULLCMD"] = ":"
   CONFIG["ENABLE_DEBUG_ENV"] = ""
   CONFIG["DLNOBJ"] = "dln.o"
@@ -121,10 +121,8 @@ module RbConfig
   CONFIG["RPATHFLAG"] = " -Wl,-rpath,%1$-s"
   CONFIG["LIBPATHFLAG"] = " -L%1$-s"
   CONFIG["LINK_SO"] = ""
-  CONFIG["ADDITIONAL_DLDFLAGS"] = ""
   CONFIG["ASMEXT"] = "S"
   CONFIG["LIBEXT"] = "a"
-  CONFIG["DLEXT2"] = ""
   CONFIG["DLEXT"] = "so"
   CONFIG["LDSHAREDXX"] = "$(CXX) -shared $(CCDLFLAGS)"
   CONFIG["LDSHARED"] = "$(CC) -shared $(CCDLFLAGS)"
@@ -133,10 +131,9 @@ module RbConfig
   CONFIG["ARCH_FLAG"] = ""
   CONFIG["DLDFLAGS"] = "-L/usr/local/lib"
   CONFIG["ALLOCA"] = ""
-  CONFIG["EGREP"] = "/usr/bin/egrep"
-  CONFIG["GREP"] = "/usr/bin/grep"
   CONFIG["dsymutil"] = ""
   CONFIG["codesign"] = ""
+  CONFIG["cleanlibs"] = ""
   CONFIG["POSTLINK"] = ":"
   CONFIG["WERRORFLAG"] = "-Werror"
   CONFIG["CHDIR"] = "cd -P"
@@ -150,9 +147,9 @@ module RbConfig
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.0.6/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.0.6/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.0.6/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.1.1/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.1.1/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.1.1/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
@@ -169,12 +166,14 @@ module RbConfig
   CONFIG["CPPOUTFILE"] = "-o conftest.i"
   CONFIG["GNU_LD"] = "yes"
   CONFIG["GCC"] = "yes"
+  CONFIG["EGREP"] = "/usr/bin/egrep"
+  CONFIG["GREP"] = "/usr/bin/grep"
   CONFIG["CPP"] = "$(CC) -E"
-  CONFIG["CFLAGS"] = "-O2 -pipe -fPIC"
+  CONFIG["CXXFLAGS"] = "-O2 -pipe"
   CONFIG["OBJEXT"] = "o"
   CONFIG["CPPFLAGS"] = "-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include $(DEFS) $(cppflags)"
   CONFIG["LDFLAGS"] = "-L. -L/usr/local/lib -fstack-protector-strong -Wl,-E"
-  CONFIG["CXXFLAGS"] = "-O2 -pipe"
+  CONFIG["CFLAGS"] = "-O2 -pipe -fPIC"
   CONFIG["STRIP"] = "strip -S -x"
   CONFIG["RANLIB"] = "ranlib"
   CONFIG["OBJDUMP"] = "objdump"
@@ -185,18 +184,18 @@ module RbConfig
   CONFIG["AS"] = "as"
   CONFIG["AR"] = "ar"
   CONFIG["CC"] = "cc -std=gnu99"
-  CONFIG["target_os"] = "openbsd7.4"
+  CONFIG["target_os"] = "openbsd7.1"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "sparc64"
-  CONFIG["target"] = "sparc64-unknown-openbsd7.4"
-  CONFIG["host_os"] = "openbsd7.4"
+  CONFIG["target"] = "sparc64-unknown-openbsd7.1"
+  CONFIG["host_os"] = "openbsd7.1"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "sparc64"
-  CONFIG["host"] = "sparc64-unknown-openbsd7.4"
-  CONFIG["build_os"] = "openbsd7.4"
+  CONFIG["host"] = "sparc64-unknown-openbsd7.1"
+  CONFIG["build_os"] = "openbsd7.1"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "sparc64"
-  CONFIG["build"] = "sparc64-unknown-openbsd7.4"
+  CONFIG["build"] = "sparc64-unknown-openbsd7.1"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
@@ -235,7 +234,7 @@ module RbConfig
   CONFIG["libexecdir"] = "$(exec_prefix)/libexec"
   CONFIG["sbindir"] = "$(exec_prefix)/sbin"
   CONFIG["bindir"] = "$(exec_prefix)/bin"
-  CONFIG["program_transform_name"] = "s&$$&30&;s&^&&"
+  CONFIG["program_transform_name"] = "s&$$&31&;s&^&&"
   CONFIG["exec_prefix"] = "$(prefix)"
   CONFIG["PACKAGE_URL"] = ""
   CONFIG["PACKAGE_BUGREPORT"] = ""
@@ -245,8 +244,9 @@ module RbConfig
   CONFIG["PACKAGE_NAME"] = ""
   CONFIG["PATH_SEPARATOR"] = ":"
   CONFIG["SHELL"] = "/bin/sh"
-  CONFIG["UNICODE_VERSION"] = "12.1.0"
-  CONFIG["UNICODE_EMOJI_VERSION"] = "12.1"
+  CONFIG["UNICODE_VERSION"] = "13.0.0"
+  CONFIG["UNICODE_EMOJI_VERSION"] = "13.1"
+  CONFIG["platform"] = "$(arch)"
   CONFIG["archdir"] = "$(rubyarchdir)"
   CONFIG["topdir"] = File.dirname(__FILE__)
   # Almost same with CONFIG. MAKEFILE_CONFIG has other variable
